@@ -312,7 +312,7 @@ class Catmaid:
         
         Args:
             relation_id (str): presynaptic_to or postsynaptic_to.
-            nb_connectors (uint): number of connectors.
+            nb_connectors (uint): number of connectors. If None, then return all neurons regardess their number of connectors.
         Returns:
             dictionary whose keys are neuron_id, values are treenode_ids.
         
@@ -321,7 +321,11 @@ class Catmaid:
         subdf = self.dfcon[self.dfcon['relation_id']==relation_id].copy()
         subdf_counts = subdf.groupby(['neuron_id'])['treenode_id'].count()
         dic = {}
-        neulst = subdf_counts[subdf_counts==nb_connectors].index.values
+        if nb_connectors is None:
+            neulst = subdf_counts.index.values
+        else:
+            neulst = subdf_counts[subdf_counts==nb_connectors].index.values
+        
         for neuron_id in neulst:
             dic[neuron_id] = list(subdf[subdf["neuron_id"]==neuron_id]["treenode_id"].values)
         
@@ -334,7 +338,7 @@ class Catmaid:
         We assume innervated neuron as postsynaptic neuron, the ones who make innervation are presynaptic neurons.
                 
         Args:
-            nb_innervations (int): number innervated presynaptic neurons.
+            nb_innervations (int): number innervated presynaptic neurons. If None, then return all neurons.
             
         Returns:
             dictionary whose key is postsynaptic neuron, value is innervating presynaptic neurons.
@@ -346,7 +350,10 @@ class Catmaid:
         if self.dfsyn is not None:
             subdf = self.dfsyn.sort_values(['post_neuron_id'])
             subdf_counts = subdf.groupby(['post_neuron_id'])['pre_neuron_id'].count()
-            neulst = subdf_counts[subdf_counts==nb_innervations].index.values
+            if nb_innervations is None:
+                neulst = subdf_counts.index.values
+            else:
+                neulst = subdf_counts[subdf_counts==nb_innervations].index.values
             for neuron_id in neulst:
                 dic[neuron_id] = subdf[subdf["post_neuron_id"]==neuron_id][["pre_neuron_id","pre_treenode_id"]].values.tolist()
         
