@@ -13,7 +13,7 @@ from PyAstronomy import pyasl
 
 from sklearn import metrics
 
-from genepy3d.util.geo import active_brownian_2d, active_brownian_3d, geo_len, norm, angle3points, vector2points, vector_axes_angles, angle2vectors, l2 as distl2
+from genepy3d.util.geo import active_brownian_2d, active_brownian_3d, geo_len, norm, angle3points, vector2points, vector_axes_angles, angle2vectors
 from genepy3d.util import plot as pl
 
 from matplotlib.colors import ListedColormap
@@ -29,12 +29,9 @@ class Curve:
         
             - if ``array`` is given, then each column is the x, y, z coordinates of a specific point.
             - if ``tuple`` is given, then the items are the three arrays of x, y and z coordinates.
-        
-        curve_id (int): curve ID (optional).
-        curve_name (str): curve name (optional).
     
     """
-    def __init__(self, coors, curve_id=None, curve_name=None):
+    def __init__(self, coors):
         
         if isinstance(coors,np.ndarray):
             self.coors = coors
@@ -52,17 +49,12 @@ class Curve:
             
         
         self.nb_of_points = self.coors.shape[0]
-            
-        if curve_id is None:
-            self.curve_id = 0 # default value
-        else:
-            self.curve_id = curve_id
-        
-        if curve_name is None:
-            self.curve_name = 'GeNePy3D'
-        else:
-            self.curve_name = curve_name
         
+    
+    @property
+    def size(self):
+        return len(self.coors)
+    
     @classmethod
     def from_csv(cls,filepath,column_names=["x","y","z"],scales=(1.,1.,1.),args={}):
         """Constructor from csv file.
@@ -463,12 +455,12 @@ class Curve:
                 new_coors = np.array([xi,yi,zi]).T
         
         if return_interp_param==False:
-            return Curve(new_coors,self.curve_id,self.curve_name)
+            return Curve(new_coors)
         else:
             try:
-                return (Curve(new_coors,self.curve_id,self.curve_name),to,coef,ti,uix)
+                return (Curve(new_coors),to,coef,ti,uix)
             except:
-                return Curve(new_coors,self.curve_id,self.curve_name)
+                return Curve(new_coors)
     
     def denoise(self,return_std=False):
         """Automatically denoise a curve assuming Gaussian noise. Noise variance acrooss the three dimensions is estimated using the beta-sigma method implemented in PyAstronomy [1]. 
