@@ -18,12 +18,19 @@ class Points:
     """Points cloud in 3D.
     
     Attributes:
-        coors (2d array): list of 3d points.
+        coors (array | tuple): list of 3d points.
+            - if ``array`` is given, then each column is the x, y, z coordinates of a specific point.
+            - if ``tuple`` is given, then the items are the three arrays of x, y and z coordinates.
+        
         size (int): number of points
     
     """
     def __init__(self, _coors):
-        self.coors = _coors
+        
+        if isinstance(_coors,np.ndarray):
+            self.coors = _coors
+        elif isinstance(_coors,(tuple,list)):
+            self.coors = np.array(_coors).T
     
     @property
     def size(self):
