@@ -29,6 +29,10 @@ class Surface(trimesh.Trimesh):
         
         Returns:
             a Surface object
+
+        Note:
+            Aware of using this func since the volume computed from this surface is different from one of Trimesh (see convex_hull()).
+
         """               
 
         from scipy.spatial import ConvexHull
@@ -70,10 +74,21 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
         Dy = -np.linalg.det(np.concatenate((normsq,tetrapos[:,:,[0,2]],ones),axis=2))
         Dz = np.linalg.det(np.concatenate((normsq,tetrapos[:,:,[0,1]],ones),axis=2))
         c = np.linalg.det(np.concatenate((normsq,tetrapos),axis=2))
+        
+        # Remove zero determinant
+        ids = np.argwhere(a!=0).flatten()
+        Dx = Dx[ids]
+        Dy = Dy[ids]
+        Dz = Dz[ids]
+        a = a[ids]
+        c = c[ids]
+        tetra_vertices = tetra.vertices[ids]
+        
+        # Radius of circumshpere
         r = np.sqrt(Dx**2+Dy**2+Dz**2-4*a*c)/(2*np.abs(a))
-
+        
         # Find tetrahedrals
-        tetras = tetra.vertices[r<alpha,:]
+        tetras = tetra_vertices[r<alpha,:]
         # triangles
         TriComb = np.array([(0, 1, 2), (0, 1, 3), (0, 2, 3), (1, 2, 3)])
         Triangles = tetras[:,TriComb].reshape(-1,3)
@@ -107,8 +122,8 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
 #        return
     
     @classmethod
-    def from_STL(cls,afile,xy2zratio=None,center=False): 
-        """Create surface object from STL file. Does no check on the loaded mesh.
+    def from_stl(cls,afile,xy2zratio=None,center=False): 
+        """Create surface object from STL file.
         
         Args:
             afile (string): path to load
@@ -119,18 +134,28 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
             a Surface object
         """
          
-        from stl import mesh
+        # from stl import mesh
         
-        your_mesh = mesh.Mesh.from_file(afile)
-        points=np.reshape(your_mesh.data['vectors'],[3*your_mesh.data['vectors'].shape[0],3])
-        points=np.unique(points,axis=0)
+        # your_mesh = mesh.Mesh.from_file(afile)
+        # points=np.reshape(your_mesh.data['vectors'],[3*your_mesh.data['vectors'].shape[0],3])
+        # points=np.unique(points,axis=0)
  
-        inds=[]
-        for i in range(your_mesh.data['vectors'].shape[0]):
-            for j in range(your_mesh.data['vectors'].shape[1]):
-                inds.append(np.where([(points[k,:]==your_mesh.data['vectors'][i][j]).all() for k in range(len(points))])[0])
-        inds=np.reshape(np.stack(inds),your_mesh.data['vectors'].shape[0:2])  
-        #points_centered=preprocessing.scale(points,with_std=False)    
+        # inds=[]
+        # for i in range(your_mesh.data['vectors'].shape[0]):
+        #     for j in range(your_mesh.data['vectors'].shape[1]):
+        #         inds.append(np.where([(points[k,:]==your_mesh.data['vectors'][i][j]).all() for k in range(len(points))])[0])
+        # inds=np.reshape(np.stack(inds),your_mesh.data['vectors'].shape[0:2])  
+        # #points_centered=preprocessing.scale(points,with_std=False)    
+        
+        # Load file using load() in trimesh
+        try:
+            your_mesh = trimesh.load(afile)
+        except:
+            raise Exception("Failed to load stl file.")
+        
+        points = your_mesh.vertices
+        inds = your_mesh.faces
+        
         if center:
             points[:,0]=points[:,0]-np.mean(points[:,0])
             points[:,1]=points[:,1]-np.mean(points[:,1])
@@ -141,6 +166,23 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
         
         return cls(points,inds)
     
+    @classmethod
+    def from_ply(cls,afile,xy2zratio=None,center=False): 
+        """Create surface object from PLY file.
+        
+        Args:
+            afile (string): path to load
+            xy2zratio (float): in case of anisotromic measurement, ratio to apply to z coordinates w.r.t. x and y 
+            center (bool): center the mesh
+        
+        Returns:
+            a Surface object
+        """
+
+        # Since from_STL() use trimesh.load() func, it is also convenient to load ply file
+        return cls.from_stl(afile,xy2zratio,center)
+
+    
     @classmethod
     def from_volume(cls,vol,lbl,spacing=(1.,1.,1.),step_size=1,level=0.5,opening=False,fillholes=False):
         """Create isosurface from 3D volume by marching cubes.
@@ -185,6 +227,91 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
         
         return cls(verts[:,[2,1,0]],faces)
     
+    def get_qhull(self):
+        """Compute the Convex Hull of the surface.
+        """
+
+        # from scipy.spatial import ConvexHull
+        # hull = ConvexHull(self.vertices)
+        # #becasue hull.simplices are in the context of the full point list, not just Qhull vertexes
+        # simplices=np.reshape(np.stack([np.where(hull.vertices==i)[0] for i in hull.simplices.flat]),hull.simplices.shape) 
+        
+        qhull = self.convex_hull
+        return Surface(qhull.vertices, qhull.faces)
+
+    def get_bbox(self):
+        """Compute the bounding box of the surface.
+        """
+
+        return Surface(self.bounding_box.vertices,self.bounding_box.faces)
+    
+    def get_obbox(self):
+        """Compute oriented bounding box of the surface.
+        """
+        
+        return Surface(self.bounding_box_oriented.vertices,self.bounding_box_oriented.faces)
+
+    def get_obbox_axes(self):
+        """Return axes and their lengths of the oriented bounding box. 
+
+        Returns:
+            coordinates of three axes.
+            lengths of three axes.
+        """
+
+        # Get oriented bounding box
+        obbox = self.get_obbox()
+
+        # Compute the angles between adjacent faces
+        theta = np.round(obbox.face_adjacency_angles,3)
+
+        # Find inplane edges (should be excluded from the measurement)
+        inplane_edges = obbox.face_adjacency_edges[theta==0]
+
+        # Exclude inplane edges
+        edges_axis = []
+        edges_len = []
+        for i in range(len(obbox.edges_unique)):
+            query = len(np.argwhere(np.array([len(np.setdiff1d(item,obbox.edges_unique[i])) for item in inplane_edges])==0).flatten())==0
+            if query is True: # not inplane edge
+                # print("Included",obbox.edges_unique[i],np.round(obbox.edges_unique_length[i],3))
+                edges_len.append(np.round(obbox.edges_unique_length[i],3))
+                edges_axis.append(obbox.edges_unique[i])
+            # else:
+            #     print("Excluded",obbox.edges_unique[i])
+        
+        # Get only unique edges
+        edges_len, uix = np.unique(edges_len,return_index=True)
+        edges_axis = np.array(edges_axis)[uix]
+
+        # Sort by lengths
+        six = np.argsort(edges_len)
+        edges_len = edges_len[six]
+        edges_axis = edges_axis[six]
+
+        return obbox.vertices[edges_axis], edges_len
+
+    def compute_sphericity(self):
+        """Compute the sphericity index of the surface.
+
+        Source: https://www.sciencedirect.com/science/article/pii/S1877750318304757#bib0205
+
+        Sphericity = ((minor**2) / (median*major))**(1/3),
+        where minor, median and major are lengths of the elliposoid fitting the surface.
+        These lengths can be found from the orient bounding box of the surface.
+
+        """
+
+        # Get edges lengths of the oriented bounding boxes
+        _, edges_len = self.get_obbox_axes()
+
+        if len(edges_len)!=3:
+            raise Exception("Failed to determine minor, medium, major lengths of bounding box.")
+        else:
+            sphericity_index = (edges_len[0]**2 / (edges_len[1]*edges_len[2]))**(1./3)
+        
+        return sphericity_index
+    
     def split(self):
         """Override split() by casting Trimesh to Surface.
 
@@ -197,7 +324,7 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
             for m in trimesh_splits:
                 surface_splits.append(Surface(m.vertices,m.faces))
         return surface_splits
-        
+    
     def to_points(self):
         """Convert to Points 
         
@@ -285,7 +412,7 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
         if 'args_3d' in kwds.keys():
             args_3d = kwds['args_3d']
         else:
-            args_3d = {'edgecolor':'yellow','facecolor':'yellow','alpha':0.3}
+            args_3d = {'edgecolor':'none','color':'yellow','alpha':0.5,'linewidths':0.1}
             
         if 'equal_aspect' in kwds.keys():
             equal_aspect = kwds['equal_aspect']
@@ -322,7 +449,7 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
             if equal_aspect==True:
                 ax.axis('equal')
                 
-    def _plot3d(self,ax,coors,args_3d={"edgecolor":'yellow',"facecolor":'yellow',"alpha":"0.3"}):
+    def _plot3d(self,ax,coors,args_3d={'edgecolor':'none','color':'yellow','alpha':0.5,'linewidths':0.1}):
         """Support 3d plot outline.
         
         Args:
@@ -399,5 +526,7 @@ Python implementation from https://stackoverflow.com/questions/26303878/alpha-sh
                 else:
                     ax.plot(xnew[idx],ynew[idx],c=edgecolor,lw=edgewidth,alpha=alpha)
         
+def available_formats():
+    return trimesh.available_formats()
 
         
