@@ -449,6 +449,63 @@ class Tree:
         df.set_index('nodeid',inplace=True)
         return df    
     
+    def get_features(self,features=["x","y","z"],nodeid=None,rootid=None):
+        """Return list of features from given nodes.
+        
+        Args:
+            nodes (int | list of int): list of node IDs.
+            
+        Returns:
+            Pandas dataframe whose indices are node IDs and values are corresponding features.
+        
+        """
+        
+        if rootid is None:
+            _rootid = self.get_root()[0]
+        else:
+            _rootid = rootid
+        
+        if nodeid is None:
+            nodelst = [node.name for node in PreOrderIter(self.nodes[_rootid])]
+        elif nodeid is not None:
+            if isinstance(nodeid,(int,np.integer)): # only one item.
+                nodelst = [nodeid]
+            elif isinstance(nodeid,(list,np.ndarray)):
+                nodelst = nodeid
+            else:
+                raise Exception("node_id must be array-like.")
+        
+        feats = []
+        for i in nodelst:
+            node = self.nodes[i]
+            feats.append([i]+[getattr(node,f) for f in features])
+        feats = np.array(feats)
+        column_names = ["nodeid"] + features
+        df = pd.DataFrame(feats,columns=column_names)
+        df.set_index('nodeid',inplace=True)
+        return df
+        
+    
+    def get_bbox(self,nodeid=None,rootid=None):
+        """Return bounding box positions covering the tree. 
+        
+        If nodeid is None, then bounding box of the whole tree. 
+        Otherwise, bounding box of the given set of nodes.
+        
+        Args:
+            nodeid (list[int]): list of node IDs or None. (default None)
+        
+        Returns:
+            List of [(Xmin,Xmax),(Ymin,Ymax),(Zmin,Zmax)]
+
+        
+        """
+        coors = self.get_coordinates(nodeid,rootid)
+        xmin, xmax = coors['x'].min(), coors['x'].max()
+        ymin, ymax = coors['y'].min(), coors['y'].max()
+        zmin, zmax = coors['z'].min(), coors['z'].max()
+        return [(xmin,xmax),(ymin,ymax),(zmin,zmax)]
+    
     def compute_length(self,nodeid=None,rootid=None):
         """Return length of the tree of part of it. If nodeid is `none`, compute length of the whole tree. Else compute the length of the given set of nodes, assuming they are contiguous and ordered.
         
@@ -1486,7 +1543,7 @@ class Tree:
         return Points(self.get_coordinates(nodeid).values)
     
     def plot(self,ax,projection='3d',rootid=None,spine_only=False,
-             show_root=True,show_leaves=True,show_branchingnodes=True,show_connectors=True,
+             show_root=True,show_leaves=True,show_branchingnodes=True,show_connectors=True,show_nodes=False,
              root_args={},leaves_args={},branchingnodes_args={},connectors_args={},
              weights=None,weights_display_type="c",point_args = {},show_cbar=False,cbar_args={},
              line_args={},scales=(1.,1.,1.),equal_axis=True):
@@ -1500,6 +1557,7 @@ class Tree:
             show_leaves (bool): if True, then display leaves nodes.
             show_branchingnodes (bool): if True, then display branching nodes.
             show_connectors (bool): if True, then display connector nodes.
+            show_nodes (bool): if True, then display all nodes.
             root_args (dic): plot params for root node.
             leaves_args (dic): plot params for leaves nodes.
             branchingnodes_args (dic): plot params for branching nodes.
@@ -1551,6 +1609,11 @@ class Tree:
             x, y, z = coors[:,0], coors[:,1], coors[:,2]
             pl.plot_point(ax,projection,x,y,z,scales,_root_args)
         
+        if show_nodes==True:
+            coors = self.get_coordinates().values
+            x, y, z = coors[:,0], coors[:,1], coors[:,2]
+            pl.plot_point(ax,projection,x,y,z,scales,_point_args)
+            
         if spine_only==True:
             
             spine_nodes = self.compute_spine(_rootid)
